# -*- coding: UTF-8 -*-
'''
    magento.catalog

    Product Catalog API for magento

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD
    :license: AGPLv3, see LICENSE for more details
'''

import base64
from mimetypes import guess_type

from magento.api import API


class Category(API):
    """
    Product Category API
    """
    __slots__ = ( )

    def currentStore(self, store_view=None):
        """
        Set/Get current store view

        :param store_view: Store view ID or Code
        :return: int
        """
        args = [store_view] if store_view else []
        return int(self.call('catalog_category.currentStore', args))

    def tree(self, parent_id=None, store_view=None):
        """
        Retrieve hierarchical tree of categories.

        :param parent_id: Integer ID of parent category (optional)
        :param store_view: Store View (optional)
        :return: dictionary of values
        """
        return self.call('catalog_category.tree', [parent_id, store_view])

    def level(self, website=None, store_view=None, parent_category=None):
        """
        Retrieve one level of categories by website/store view/parent category

        :param website: Website code or ID
        :param store_view: storeview code or ID
        :param parent_category: Parent Category ID
        :return: Dictionary
        """
        return self.call(
            'catalog_category.level', [website, store_view, parent_category]
        )

    def info(self, category_id, store_view=None, attributes=None):
        """
        Retrieve Category details

        :param category_id: ID of category to retrieve
        :param store_view: Store view ID or code
        :param attributes: Return the fields specified
        :return: Dictionary of data
        """
        return self.call(
            'catalog_category.info', [category_id, store_view, attributes]
        )

    def create(self, parent_id, data, store_view=None):
        """
        Create new category and return its ID

        :param parent_id: ID of parent
        :param data: Data for category
        :param store_view: Store view ID or Code
        :return: Integer ID
        """
        return int(self.call(
            'catalog_category.create', [parent_id, data, store_view])
        )

    def update(self, category_id, data, store_view=None):
        """
        Update Category

        :param category_id: ID of category
        :param data: Category Data
        :param store_view: Store view ID or code
        :return: Boolean
        """
        return bool(
            self.call(
                'catalog_category.update', [category_id, data, store_view]
            )
        )

    def move(self, category_id, parent_id, after_id=None):
        """
        Move category in tree

        :param category_id: ID of category to move
        :param parent_id: New parent of the category
        :param after_id: Category ID after what position it will be moved
        :return: Boolean
        """
        return bool(self.call(
            'catalog_category.move', [category_id, parent_id, after_id])
        )

    def delete(self, category_id):
        """
        Delete category

        :param category_id: ID of category
        :return: Boolean
        """
        return bool(self.call('catalog_category.delete', [category_id]))

    def assignedproducts(self, category_id, store):
        """
        Retrieve list of assigned products

        :param category_id: Category ID
        :param store: Store ID or Code

        :return: Dictionary
        """
        return self.call(
            'catalog_category.assignedProducts', [category_id, store]
        )

    #: A proxy for :meth:`assignedproducts`
    assigned_products = assignedproducts

    def assignproduct(self, category_id, product, position=None):
        """
        Assign product to a category

        :param category_id: ID of a category
        :param product: ID or Code of the product
        :param position: Position of product in category

        :return: boolean
        """
        return bool(self.call(
            'catalog_category.assignProduct', [category_id, product, position])
        )

    #: A proxy for :meth:`assignproduct`
    assign_product = assignproduct

    def updateproduct(self, category_id, product, position=None):
        """
        Update assigned product

        :param category_id: ID of a category
        :param product: ID or Code of the product
        :param position: Position of product in category

        :return: boolean
        """
        return bool(self.call(
            'catalog_category.updateProduct', [category_id, product, position])
        )

    #: A proxy for :meth:`updateproduct`
    update_product = updateproduct

    def removeproduct(self, category_id, product):
        """
        Remove product from category

        :param category_id: ID of a category
        :param product: ID or Code of the product

        :return: boolean
        """
        return bool(self.call(
            'catalog_category.removeProduct', [category_id, product])
        )

    #: A proxy for :meth:`removeproduct`
    remove_product = removeproduct


class CategoryAttribute(API):
    """
    Product Category Attribute API to connect to magento
    Allows to get attributes and options for category.
    """
    __slots__ = ( )

    def currentStore(self, store_view=None):
        """
        Set/Get current store view

        :param store_view: Store view ID or Code
        :return: int
        """
        args = [store_view] if store_view else []
        return int(self.call('catalog_category_attribute.currentStore', args))

    def list(self):
        """
        Retrieve Category attrbutes
        """
        return self.call('category_attribute.list', [])

    def options(self, attribute_id, store_view=None):
        """
        Retrieve attribute options

        :param attribute_id: ID of the attribute whose options are reqd
        :param store_view: ID or Code of the store view

        :return: list of dictionary
        """
        return self.call(
            'category_attribute.options', [attribute_id, store_view]
        )


class Product(API):
    """
    Product API for magento
    """
    __slots__ = ( )

    def currentStore(self, store_view=None):
        """
        Set/Get current store view

        :param store_view: Store view ID or Code
        :return: int
        """
        args = [store_view] if store_view else []
        return int(self.call('catalog_product.currentStore', args))

    def list(self, filters=None, store_view=None):
        """
        Retrieve product list by filters

        :param filters: Dictionary of filters.

               Format :
                   `{<attribute>:{<operator>:<value>}}`
               Example :
                   `{'firstname':{'ilike':'sharoon'}}`

        :param store_view: Code or ID of store view
        :return: `list` of `dict`
        """
        return self.call('catalog_product.list', [filters, store_view])

    def info(self, product, store_view=None, attributes=None):
        """
        Retrieve product data

        :param product: ID or SKU of product
        :param store_view: ID or Code of store view
        :param attributes: List of fields required
        :return: `dict` of values
        """
        return self.call(
            'catalog_product.info', [product, store_view, attributes]
        )

    def create(self, product_type, attribute_set_id, sku, data):
        """
        Create Product and return ID

        :param product_type: String type of product
        :param attribute_set_id: ID of attribute set
        :param sku: SKU of the product
        :param data: Dictionary of data
        :return: INT id of product created
        """
        return int(self.call(
            'catalog_product.create', 
            [product_type, attribute_set_id, sku, data]
            )
        )

    def update(self, product, data, store_view=None):
        """
        Update product Information

        :param product: ID or SKU of product
        :param data: Dictionary of attributes to update
        :param store_view: ID or Code of store view

        :return: Boolean
        """
        return bool(self.call(
            'catalog_product.update', [product, data, store_view])
        )

    def setSpecialPrice(self, product, special_price=None,
                        from_date=None, to_date=None, store_view=None):
        """
        Update product's special price

        :param product: ID or SKU of product
        :param special_price: Special Price
        :param from_date: From date
        :param to_date: To Date
        :param store_view: ID or Code of Store View

        :return: Boolean
        """
        return bool(self.call(
            'catalog_product.setSpecialPrice', 
            [product, special_price, from_date, to_date, store_view]
            )
        )

    def getSpecialPrice(self, product, store_view=None):
        """
        Get product special price data

        :param product: ID or SKU of product
        :param store_view: ID or Code of Store view

        :return: Dictionary
        """
        return self.call(
            'catalog_product.getSpecialPrice', [product, store_view]
        )

    def delete(self, product):
        """
        Delete a product

        :param product: ID or SKU of product
        :return: Boolean
        """
        return bool(self.call('catalog_product.delete', [product]))


class ProductAttribute(API):
    """
    Product Attribute API
    """
    __slots__ = ( )

    def currentStore(self, store_view=None):
        """
        Set/Get current store view

        :param store_view: Store view ID or Code
        :return: int
        """
        args = [store_view] if store_view else []
        return int(self.call('catalog_product_attribute.currentStore', args))

    def list(self, attribute_set_id):
        """
        Retrieve product attribute list

        :param attribute_set_id: ID of attribute set
        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute.list', [attribute_set_id])

    def options(self, attribute, store_view=None):
        """
        Retrieve product attribute options

        :param attribute: ID or Code of the attribute
        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute.options',
                [attribute, store_view])

class ProductAttributeSet(API):
    """
    Product Attribute Set API
    """
    __slots__ = ( )

    def list(self):
        """
        Retrieve list of product attribute sets

        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute_set.list', [ ])


class ProductTypes(API):
    """
    Product Types API
    """
    __slots__ = ( )

    def list(self):
        """
        Retrieve list of product types

        :return: `list` of `dict`
        """
        return self.call('catalog_product_type.list', [ ])


class ProductImages(API):
    """
    Product Images API
    """
    __slots__ = ( )

    def currentStore(self, store_view=None):
        """
        Set/Get current store view

        :param store_view: Store view ID or Code
        :return: int
        """
        args = [ ]
        if store_view:
            args = [store_view]
        return int(self.call('catalog_product_attribute_media.currentStore',
            args))

    def list(self, product, store_view=None):
        """
        Retrieve product image list

        :param product: ID or SKU of product
        :param store_view: Code or ID of store view
        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute_media.list',
                [product, store_view])

    def info(self, product, image_file, store_view=None):
        """
        Retrieve product image data

        :param product: ID or SKU of product
        :param store_view: ID or Code of store view
        :param attributes: List of fields required
        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute_media.info',
                [product, image_file, store_view])

    def types(self, attribute_set_id):
        """
        Retrieve product image types (image, small_image, thumbnail, etc.)

        :param attribute_set_id: ID of attribute set
        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute_media.types',
                [attribute_set_id])

    def create(self, product, image, image_name, image_mime=None,
            store_view=None):
        """
        Upload a new product image.

        :param product: ID or SKU of product
        :param image: The binary data of image
        :param image_name: The name to give the image file. 
                Eg 'my_image_thumb.jpg'
        :param image_mime: The mime type of the image.
        :param store_view: Store view ID or Code
        :return: string - image file name
        """
        if image_mime is None:
            image_mime = guess_type(image_name)
        data = {
            'file': {
                    'content': base64.b64encode(image),
                    'name': image_name,
                    'mime': image_mime,
                        }
                }
        return self.call('catalog_product_attribute_media.create',
                [product, data, store_view])

    def update(self, product, img_file_name, data, store_view=None):
        """
        Update a product image.

        :param product: ID or SKU of product
        :param img_file_name: The image file name
            Example: '/m/y/my_image_thumb.jpg'
        :param data: `dict` of image data (label, position, exclude, types)
            Example: { 'label': 'description of photo',
                'position': '1', 'exclude': '0',
                'types': ['image', 'small_image', 'thumbnail']}
        :param store_view: Store view ID or Code
        :return: string - image file name
        """
        return self.call('catalog_product_attribute_media.update',
                [product, img_file_name, data, store_view])

    def remove(self, product, img_file_name):
        """
        Remove a product image.

        :param product: ID or SKU of product
        :param img_file_name: The image file name
            Example: '/m/y/my_image_thumb.jpg'
        :return: boolean
        """
        return self.call('catalog_product_attribute_media.remove',
                [product, img_file_name])


class ProductTierPrice(API):
    """
    Product Tier Price API
    """
    __slots__ = ( )

    def info(self, product):
        """
        Retrieve product data

        :param product: ID or SKU of product
        :return: `list` of `dict`
        """
        return self.call('catalog_product_attribute_tier_price.info', [product])

    def update(self, product, data):
        """
        Update product tier prices.

        Note: All existing tier prices for the product are replaced by the tier
        prices provided in data.

        :param product: ID or SKU of product
        :param data: List of dictionaries of tier price information
            Example:
                [{
                    'website': 'all',
                    'customer_group_id': '1',
                    'qty': '99.0000',
                    'price': '123.9900'
                },
                {
                    'website': 'all',
                    ...
                },...]

        :return: Boolean
        """
        return bool(self.call('catalog_product_attribute_tier_price.update',
                [product, data]))


class ProductLinks(API):
    """
    Product links API (related, cross sells, up sells, grouped)
    """
    __slots__ = ( )

    def list(self, link_type, product):
        """
        Retrieve list of linked products

        :param link_type: type of link, one of 'cross_sell', 'up_sell',
                'related' or 'grouped'
        :param product: ID or SKU of product
        :return: `list` of `dict`
        """
        return self.call('catalog_product_link.list', [link_type, product])

    def assign(self, link_type, product, linked_product, data=None):
        """
        Assign a product link

        :param link_type: type of link, one of 'cross_sell', 'up_sell',
                'related' or 'grouped'
        :param product: ID or SKU of product
        :param linked_product: ID or SKU of linked product
        :param data: dictionary of link data, (position, qty, etc.)
                Example: { 'position': '0', 'qty': 1}
        :return: boolean
        """
        return bool(self.call('catalog_product_link.assign',
                [link_type, product, linked_product, data]))

    def update(self, link_type, product, linked_product, data=None):
        """
        Update a product link

        :param link_type: type of link, one of 'cross_sell', 'up_sell',
                'related' or 'grouped'
        :param product: ID or SKU of product
        :param linked_product: ID or SKU of linked product
        :param data: dictionary of link data, (position, qty, etc.)
                Example: { 'position': '0', 'qty': 1}
        :return: boolean
        """
        return bool(self.call('catalog_product_link.update',
                [link_type, product, linked_product, data]))

    def remove(self, link_type, product, linked_product):
        """
        Remove a product link

        :param link_type: type of link, one of 'cross_sell', 'up_sell',
                'related' or 'grouped'
        :param product: ID or SKU of product
        :param linked_product: ID or SKU of linked product to unlink
        :return: boolean
        """
        return bool(self.call('catalog_product_link.remove',
                [link_type, product, linked_product]))

    def types(self):
        """
        Retrieve a list of product link types

        :return: `list` of types
        """
        return self.call('catalog_product_link.types', [ ])

    def attributes(self, link_type):
        """
        Retrieve a list of attributes of a product link type

        :param link_type: type of link, one of 'cross_sell', 'up_sell',
                'related' or 'grouped'
        :return: `list` of `dict`
               Format :
                   `[{'code': <attribute code>, 'type': <attribute type>}, ...]`
               Example :
                   `[{'code': 'position', 'type': 'int'},
                     {'code': 'qty', 'type': 'decimal'}]`
        """
        return self.call('catalog_product_link.attributes', [link_type])


class Inventory(API):
    """
    Allows to update stock attributes (status, quantity)
    """
    __slots__ = ( )

    def list(self, products):
        """
        Retrieve inventory stock data by product ids

        :param products: list of IDs or SKUs of products
        :return: `list` of `dict`
        """
        return self.call('cataloginventory_stock_item.list', [products])

    def update(self, product, data):
        """
        Update inventory stock data

        :param product: ID or SKU of product
        :param data: Dictionary of data to change,
            eg dict(qty=99, is_in_stock='1')

        :return: boolean
        """
        return bool(
            self.call(
                'cataloginventory_stock_item.update', 
                [product, data]
                )
            )



# -*- coding: UTF-8 -*-
'''
    magento API

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD
    :license: AGPLv3, see LICENSE for more details
'''
__version__ = '0.1.4'

__all__ = [
            'API',
            'Customer', 'CustomerGroup', 'CustomerAddress',
            'Country', 'Region',
            'Category', 'CategoryAttribute', 'Product', 'ProductAttribute',
            'ProductAttributeSet', 'ProductTypes', 'ProductImages',
            'ProductTierPrice', 'ProductLinks', 'Inventory',
            'Order', 'Shipment', 'Invoice',
            ]

from api import API
from customer import Customer, CustomerGroup, CustomerAddress
from directory import Country, Region
from catalog import Category, CategoryAttribute
from catalog import Product, ProductAttribute, ProductAttributeSet
from catalog import ProductTypes, ProductImages, ProductTierPrice
from catalog import ProductLinks, Inventory
from sales import Order, Shipment, Invoice



# -*- coding: UTF-8 -*-
'''
    magento.api

    Generic API for magento

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD
    :license: AGPLv3, see LICENSE for more details
'''

PROTOCOLS = []
try:
    from xmlrpclib import ServerProxy
except ImportError:
    pass
else:
    PROTOCOLS.append('xmlrpc')

try:
    from suds.client import Client
except ImportError:
    pass
else:
    PROTOCOLS.append('soap')

from magento.utils import expand_url


class API(object):
    """
    Generic API to connect to magento
    """
    __slots__ = (
        'url',
        'username',
        'password',
        'session',
        'client',
        'protocol',
    )

    def __init__(self, url, username, password,
                 version='1.3.2.4', full_url=False, protocol='xmlrpc'):
        """
        This is the Base API class which other APIs have to subclass. By
        default the inherited classes also get the properties of this
        class which will allow the use of the API with the `with` statement

        A typical example to extend the API for your subclass is given below::

           from magento.api import API

            class Core(API):

                __slots__ = ( )

                def websites(self):
                    return self.call('ol_websites.list', [])

                def stores(self):
                    return self.call('ol_groups.list', [])

                def store_views(self):
                    return self.call('ol_storeviews.list', [])

        The above real life example extends the API for the custom API
        implementation for the magento extension

            magento-community/Openlabs_OpenERPConnector

        Example usage ::

            from magento.api import API

            with API(url, username, password) as magento_api:
                return magento_api.call('customer.list', [])

        .. note:: Python with statement has to be imported from __future__
        in older versions of python. *from __future__ import with_statement*

        If you want to use the API as a normal class, then you have to manually
        end the session. A typical example is below::

            from magento.api import API

            api = API(url, username, password)
            api.connect()
            try:
                return api.call('customer.list', [])
            finally:
                api.client.endSession(api.session)

        :param url: URL to the magento instance.
                    By default the URL is treated as a base url
                    of the domain to which the api part of the URL
                    is added. If you want to specify the complete
                    URL, set the full_url flag as True.
        :param username: API username of the Web services user. Note
                    that this is NOT magento admin username
        :param password: API password of the Web services user.
        :param version: The version of magento the connection is being made to.
                        It is recommended to specify this as there could be
                        API specific changes in certain calls. Default value is
                        1.3.2.4
        :param full_url: If set to true, then the `url` is expected to
                    be a complete URL
        :param protocol: 'xmlrpc' and 'soap' are valid values
        """
        assert protocol \
            in PROTOCOLS, "protocol must be %s" % ' OR '.join(PROTOCOLS)
        self.url = full_url and url or expand_url(url, protocol)
        self.username = username
        self.password = password
        self.protocol = protocol
        self.session = None
        self.client = None

    def connect(self):
        """
        Connects to the service
        but does not login. This could be used as a connection test
        """
        if self.protocol == 'xmlrpc':
            self.client = ServerProxy(self.url, allow_none=True)
        else:
            self.client = Client(self.url)

    def __enter__(self):
        """
        Entry point for with statement
        Logs in and creates a session
        """
        if self.client is None:
            self.connect()
        if self.protocol == 'xmlrpc':
            self.session = self.client.login(
                self.username, self.password)
        else:
            self.session = self.client.service.login(
                self.username, self.password)
        return self

    def __exit__(self, type, value, traceback):
        """
        Exit point

        Closes session with magento
        """
        if self.protocol == 'xmlrpc':
            self.client.endSession(self.session)
        else:
            self.client.service.endSession(self.session)
        self.session = None

    def call(self, resource_path, arguments):
        """
        Proxy for SOAP call API
        """
        if self.protocol == 'xmlrpc':
            return self.client.call(self.session, resource_path, arguments)
        else:
            return self.client.service.call(
                self.session, resource_path, arguments)

    def multiCall(self, calls, options):
        """
        Proxy for multicalls
        """
        if self.protocol == 'xmlrpc':
            return self.client.multiCall(self.session, calls, options)
        else:
            return self.client.service.multiCall(self.session, calls, options)



# -*- coding: UTF-8 -*-
'''
    magento.utils

    General purpose utility functions

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD.
    :license: AGPLv3, see LICENSE for more details
'''

def expand_url(url, protocol):
    """
    Expands the given URL to a full URL by adding
    the magento soap/wsdl parts

    :param url: URL to be expanded
    :param service: 'xmlrpc' or 'soap'
    """
    if protocol == 'soap':
        ws_part = 'api/?wsdl'
    else:
        ws_part = 'index.php/api/xmlrpc'
    return url.endswith('/') and url + ws_part or url + '/' + ws_part




# -*- coding: UTF-8 -*-
'''
    magento.sales

    Allows to export/import sales orders from/into Magento, 
    to create invoices, shipments, credit memos

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD
    :license: AGPLv3, see LICENSE for more details
'''
from .api import API


class Order(API):
    """
    Allows to import/export orders.
    """
    __slots__ = ( )

    def list(self, filters=None):
        """
        Retrieve order list by filters

        :param filters: Dictionary of filters.

               Format :
                   `{<attribute>:{<operator>:<value>}}`
               Example :
                   `{'firstname':{'ilike':'sharoon'}}`

        :return: `list` of `dict`
        """
        return self.call('sales_order.list', [filters])

    def info(self, order_increment_id):
        """
        Retrieve order info

        :param order_increment_id: Order ID
        """
        return self.call(
            'sales_order.info', [order_increment_id]
            )

    def addcomment(self, order_increment_id,
            status, comment=None, notify=False):
        """
        Add comment to order or change its state

        :param order_increment_id: Order ID
        TODO: Identify possible values for status
        """
        if comment is None:
            comment = u""
        return bool(self.call(
                'sales_order.addComment',
                [order_increment_id, status, comment, notify]
            )
        )

    #: A proxy for :meth:`addcomment`
    addComment = addcomment

    def hold(self, order_increment_id):
        """
        Hold order

        :param order_increment_id: Order ID
        """
        return bool(self.call('sales_order.hold', [order_increment_id]))

    def unhold(self, order_increment_id):
        """
        Unhold order

        :param order_increment_id: Order ID
        """
        return bool(self.call('sales_order.unhold', [order_increment_id]))

    def cancel(self, order_increment_id):
        """
        Cancel an order

        :param order_increment_id: Order ID
        """
        return bool(self.call('sales_order.cancel', [order_increment_id]))


class Shipment(API):
    """
    Allows create/export order shipments.
    """
    __slots__ = ( )

    def list(self, filters=None):
        """
        Retrieve shipment list by filters

        :param filters: Dictionary of filters.

               Format :
                   `{<attribute>:{<operator>:<value>}}`
               Example :
                   `{'firstname':{'ilike':'sharoon'}}`

        :return: `list` of `dict`
        """
        return self.call('sales_order_shipment.list', [filters])

    def info(self, shipment_increment_id):
        """
        Retrieve shipment info

        :param shipment_increment_id: Order ID
        """
        return self.call('sales_order_shipment.info', [shipment_increment_id])

    def create(self, order_increment_id, 
            items_qty, comment=None, email=True, include_comment=False):
        """
        Create new shipment for order

        :param order_increment_id: Order Increment ID
        :type order_increment_id: str
        :param items_qty: items qty to ship
        :type items_qty: associative array (order_item_id ⇒ qty) as dict
        :param comment: Shipment Comment
        :type comment: str
        :param email: send e-mail flag (optional)
        :type email: bool
        :param include_comment: include comment in e-mail flag (optional)
        :type include_comment: bool
        """
        if comment is None:
            comment = u''
        return self.call(
            'sales_order_shipment.create', [
                order_increment_id, items_qty, comment, email, include_comment
            ]
        )

    def addcomment(self, shipment_increment_id,
            comment, email=True, include_in_email=False):
        """
        Add new comment to shipment

        :param shipment_increment_id: Shipment ID
        """
        return bool(
            self.call(
                'sales_order_shipment.addComment', 
                [shipment_increment_id, comment, email, include_in_email]
            )
        )

    #: A proxy for :meth:`addcomment`
    addComment = addcomment

    def addtrack(self, shipment_increment_id, carrier, title, track_number):
        """
        Add new tracking number

        :param shipment_increment_id: Shipment ID
        :param carrier: Carrier Code
        :param title: Tracking title
        :param track_number: Tracking Number
        """
        return self.call(
            'sales_order_shipment.addTrack',
            [shipment_increment_id, carrier, title, track_number]
        )

    #: A proxy for :meth:`addtrack`
    addTrack = addtrack

    def removetrack(self, shipment_increment_id, track_id):
        """
        Remove tracking number

        :param shipment_increment_id: SHipment ID
        :param track_id: Tracking number to remove
        """
        return bool(
            self.call(
                'sales_order_shipment.removeTrack',
                [shipment_increment_id, track_id]
            )
        )

    #: A proxy for :meth:`removetrack`
    removeTrack = removetrack

    def getcarriers(self, order_increment_id):
        """
        Retrieve list of allowed carriers for order

        :param order_increment_id: Order ID
        """
        return bool(
            self.call(
                'sales_order_shipment.getCarriers', [order_increment_id]
            )
        )

    #: A proxy for :meth:`getcarriers`
    getCarriers = getcarriers


class Invoice(API):
    """
    Allows create/export order invoices
    """
    __slots__ = ( )

    def list(self, filters=None):
        """
        Retrieve invoice list by filters

        :param filters: Dictionary of filters.

               Format :
                   `{<attribute>:{<operator>:<value>}}`
               Example :
                   `{'firstname':{'ilike':'sharoon'}}`

        :return: `list` of `dict`
        """
        return self.call('sales_order_invoice.list', [filters])

    def info(self, invoice_increment_id):
        """
        Retrieve invoice info

        :param invoice_increment_id: Invoice ID
        """
        return self.call(
            'sales_order_invoice.info', [invoice_increment_id]
        )

    def create(self, order_increment_id, items_qty, 
            comment=None, email=True, include_comment=False):
        """
        Create new invoice for order

        :param order_increment_id: Order increment ID
        :type order_increment_id: str
        :param items_qty: Items quantity to invoice
        :type items_qty: dict
        :param comment: Invoice Comment
        :type comment: str
        :param email: send invoice on e-mail
        :type email: bool
        :param include_comment: Include comments in email
        :type include_comment: bool

        :rtype: str
        """
        return self.call(
            'sales_order_invoice.create', 
            [order_increment_id, items_qty, comment, email, include_comment]
        )

    def addcomment(self, invoice_increment_id,
            comment=None, email=False, include_comment=False):
        """
        Add comment to invoice or change its state

        :param invoice_increment_id: Invoice ID
        """
        if comment is None:
            comment = u""
        return bool(
            self.call(
                'sales_order_invoice.addComment',
                [invoice_increment_id, comment, email, include_comment]
            )
        )

    #: Add a proxy for :meth:`addcomment`
    addComment = addcomment

    def capture(self, invoice_increment_id):
        """
        Capture Invoice

        :attention: You should check the invoice to see if can be 
        captured before attempting to capture an invoice, otherwise 
        the API call with generate an error.

        Invoices have states as defined in the model 
        Mage_Sales_Model_Order_Invoice:

        STATE_OPEN = 1
        STATE_PAID = 2
        STATE_CANCELED = 3

        Also note there is a method call in the model that checks this 
        for you canCapture(), and it also verifies that the payment is 
        able to be captured, so the invoice state might not be the only 
        condition that’s required to allow it to be captured.

        :param invoice_increment_id: Invoice ID
        :rtype: bool
        """
        return bool(
            self.call('sales_order_invoice.capture', [invoice_increment_id])
        )

    def void(self, invoice_increment_id):
        """
        Void an invoice

        :param invoice_increment_id: Invoice ID
        :rtype: bool
        """
        return bool(
            self.call('sales_order_invoice.void', [invoice_increment_id])
        )

    def cancel(self, invoice_increment_id):
        """
        Cancel invoice

        :param invoice_increment_id: Invoice ID
        :rtype: bool
        """
        return bool(
            self.call('sales_order_invoice.cancel', [invoice_increment_id])
        )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
    Magento API

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD
    :license: AGPLv3, see LICENSE for more details

    A simple to use python library to access the magento API and
    covered by a complete test suite based on Nose tests. Also
    includes a nose plugin to do the tests based on a config
    file.

    Example usage::
    
        from magento import Customer
        url = 'http://yourmagento.com'
        apiuser = 'apiusername'
        apipass = 'password'
        with Customer(url, apiuser, apipass) as customer_api:
            customer_api.list()

    The implemented methods and APIs are from the Core API of 
    magento documented here: 
    
    http://www.magentocommerce.com/support/magento_core_api

'''
from setuptools import setup
import magento

setup(
    name = 'magento',
    version=magento.__version__,
    url='http://projects.openlabs.co.in/trac/openlabs_magento',
    license='GNU Affero General Public License v3',
    author='Sharoon Thomas, Openlabs Technologies',
    author_email='info@openlabs.co.in',
    description='Magento Core API Client',
    long_description=__doc__,
    packages=['magento'],
    zip_safe=False,
    platforms='any',
    install_requires=[
        'suds>=0.3.9',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3', 
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)




# -*- coding: UTF-8 -*-
'''
    magento.directory

    Directory Country API for magento

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD
    :license: AGPLv3, see LICENSE for more details
'''
from magento.api import API


class Country(API):
    """
    Country API to connect to magento
    """
    __slots__ = ( )

    def list(self):
        """
        Retreive list of Countries

        :return: List of dictionaries
        """
        return self.call('directory_country.list', [])


class Region(API):
    """
    Region API to connect to magento
    """
    __slots__ = ()

    def list(self, country):
        """
        Retrieve list of regions

        :param country: Country code in ISO2 or ISO3
        :return: List of Dictionaries
        """
        return self.call('directory_region.list', [country])



# -*- coding: UTF-8 -*-
'''
    magento.customer

    Customer API for magento

    :copyright: (c) 2010 by Sharoon Thomas.
    :copyright: (c) 2010 by Openlabs Technologies & Consulting (P) LTD.

    :license: AGPLv3, see LICENSE for more details
'''
from magento.api import API


class Customer(API):
    """
    Customer API

    Example usage::

        from magento import Customer as CustomerAPI

        with CustomerAPI(url, username, password) as customer_api:
            return customer_api.list()
    """
    __slots__ = ( )

    def list(self, filters=None):
        """
        Retreive list of customers

        :param filters: Dictionary of filters.

            Format: `{<attribute>:{<operator>:<value>}}`

            Example: `{'firstname':{'ilike':'sharoon'}}`
        :return: List of dictionaries of matching records
        """
        return self.call('customer.list', filters and [filters] or [{}])

    def create(self, data):
        """
        Create a customer using the given data

        :param data: Dictionary of values
        :return: Integer ID of new record
        """
        return int(self.call('customer.create', [data]))

    def info(self, id, attributes=None):
        """
        Retrieve customer data

        :param id: ID of customer
        :param attributes: `List` of attributes needed
        """
        if attributes:
            return self.call('customer.info', [id, attributes])
        else:
            return self.call('customer.info', [id])

    def update(self, id, data):
        """
        Update a customer using the given data

        :param id: ID of the customer record to modify
        :param data: Dictionary of values
        :return: Boolean
        """
        return self.call('customer.update', [id, data])

    def delete(self, id):
        """
        Delete a customer

        :param id: ID of customer to delete
        :return: Boolean
        """
        return self.call('customer.delete', [id])


class CustomerGroup(API):
    """
    Customer Group API to connect to magento
    """
    __slots__ = ( )

    def list(self):
        """
        Retreive list of customers

        :return: List of dictionaries of matching records
        """
        return self.call('customer_group.list', [])


class CustomerAddress(API):
    """
    Customer Address API
    """
    __slots__ = ()

    def list(self, customer_id):
        """
        Retreive list of customer Addresses

        :param customer_id: ID of customer whose address needs to be fetched
        :return: List of dictionaries of matching records
        """
        return self.call('customer_address.list', [customer_id])

    def create(self, customer_id, data):
        """
        Create a customer using the given data

        :param customer_id: ID of customer, whose address is being added
        :param data: Dictionary of values (country, zip, city, etc...)
        :return: Integer ID of new record
        """
        return int(self.call('customer_address.create', [customer_id, data]))

    def info(self, id):
        """
        Retrieve customer data

        :param id: ID of customer
        """
        return self.call('customer_address.info', [id])

    def update(self, id, data):
        """
        Update a customer address using the given data

        :param id: ID of the customer address record to modify
        :param data: Dictionary of values
        :return: Boolean
        """
        return self.call('customer_address.update', [id, data])

    def delete(self, id):
        """
        Delete a customer address

        :param id: ID of address to delete
        :return: Boolean
        """
        return self.call('customer_address.delete', [id])
